import React, { useMemo, useEffect, useState } from "react";
import PropTypes from "prop-types";
import { List, ListItem, ListItemText, ListItemAvatar, Theme } from "@mui/material";
import Avatar from "@mui/material/Avatar";
import WebAssetIcon from "@mui/icons-material/WebAsset";
import DeveloperBoardIcon from "@mui/icons-material/DeveloperBoard";
import SettingsInputSvideoIcon from "@mui/icons-material/SettingsInputSvideo";
import { useTranslation } from "react-i18next";
import packageData from "../../../../package.json";
import Paper from "@mui/material/Paper";
import { useTheme } from "@mui/material/styles";
import { dtos } from "types";

const getStyles = (theme: Theme) => ({
  horizontally: {
    display: "flex",
    flexDirection: "row",
    padding: 0
  },
  vertical: {
    width: "100%"
  },
  value: {
    fontSize: "0.9rem",
    fontWeight: theme.typography.fontWeightMedium
  },
  versionAvatar: {
    backgroundColor: theme.palette.secondary.main
  }
});

type Props = {
  data: dtos.SystemVersion;
};

const SystemVersionComponent: React.FC<Props> = ({ data }) => {
  const { t } = useTranslation();
  const theme = useTheme();
  const styles = useMemo(() => getStyles(theme), [theme]);
  const [listClass, setListClass] = useState(styles.horizontally);

  useEffect(() => {
    const mediaQuery = window.matchMedia("(max-width: 800px)");

    const handleMatches = (event: MediaQueryListEvent) => {
      const cssClass: any = event.matches ? styles.vertical : styles.horizontally;
      setListClass(cssClass);
    };

    mediaQuery.addEventListener("change", handleMatches);

    // Initial check
    handleMatches(mediaQuery as unknown as MediaQueryListEvent);

    return () => {
      mediaQuery.removeEventListener("change", handleMatches);
    };
  }, [styles.horizontally, styles.vertical]);

  const lastReleaseDate = useMemo(() => {
    const format = "DD-MM-YYYY HH:mm:ss";
    const server = t("DATE_FORMAT", {
      date: {
        value: data.server.lastReleaseDate,
        format
      }
    });

    const api = t("DATE_FORMAT", {
      date: {
        value: data.api.lastReleaseDate,
        format
      }
    });

    const frontend = t("DATE_FORMAT", {
      date: {
        value: process.env.APP_DATE ?? new Date(),
        format
      }
    });

    return { server, api, frontend };
  }, [data, t]);

  return (
    <Paper variant="outlined">
      <List sx={listClass}>
        <ListItem>
          <ListItemAvatar>
            <Avatar sx={styles.versionAvatar}>
              <DeveloperBoardIcon />
            </Avatar>
          </ListItemAvatar>
          <ListItemText
            primary={
              <span style={styles.value}>
                {t("About.System.Version.Server", {
                  version: data.server.version
                })}
              </span>
            }
            secondary={t("About.System.Version.LastReleaseDate", {
              date: lastReleaseDate.server
            })}
          />
        </ListItem>
        <ListItem>
          <ListItemAvatar>
            <Avatar sx={styles.versionAvatar}>
              <SettingsInputSvideoIcon />
            </Avatar>
          </ListItemAvatar>
          <ListItemText
            primary={
              <span style={styles.value}>
                {t("About.System.Version.Api", {
                  version: data.api.version
                })}
              </span>
            }
            secondary={t("About.System.Version.LastReleaseDate", {
              date: lastReleaseDate.api
            })}
          />
        </ListItem>
        <ListItem>
          <ListItemAvatar>
            <Avatar sx={styles.versionAvatar}>
              <WebAssetIcon />
            </Avatar>
          </ListItemAvatar>
          <ListItemText
            primary={
              <span style={styles.value}>
                {t("About.System.Version.Frontend", {
                  version: process.env.APP_VERSION ?? packageData.version
                })}
              </span>
            }
            secondary={t("About.System.Version.LastReleaseDate", {
              date: lastReleaseDate.frontend
            })}
          />
        </ListItem>
      </List>
    </Paper>
  );
};

export default SystemVersionComponent;
